/*
 * snoop.h
 * Copyright (C) 2005 Florin Malita <fmalita@gmail.com>
 *
 * This file is part of Snoop, a free GNU/Linux monitoring tool.
 * See http://snoop.sourceforge.net/ for details and updates.
 *
 * Snoop is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Snoop is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _SNOOP_H_
#define _SNOOP_H_

#define DEFAULT_MAX_OPEN	0x0080
#define DEFAULT_BUFFER_SIZE	0x2000

#define SNOOP_NAME_SIZE		0x80

#define SNOOP_MINOR		0x00
#define SNOOP_MINORS		0x01

#define SNOOP_IOC_MAGIC		0xfa
#define SNOOP_IOCATTACHR	_IOW(SNOOP_IOC_MAGIC, 0x80, unsigned long)
#define SNOOP_IOCATTACHW	_IOW(SNOOP_IOC_MAGIC, 0x81, unsigned long)
#define SNOOP_IOCATTACHRW	_IOW(SNOOP_IOC_MAGIC, 0x82, unsigned long)
#define SNOOP_IOCGCOUNT		_IOR(SNOOP_IOC_MAGIC, 0x83, unsigned long)

#define SNOOP_MODE_READ		0x01
#define SNOOP_MODE_WRITE	0x02

#ifdef SNOOP_DEBUG
#define TRACE(x...)	do { printk(KERN_INFO "(%s) - ", __func__); printk(x); } while(0)
#else
#define TRACE(x...)	do {} while(0)
#endif

#define ERROR(x...)	do { printk(KERN_ERR "(%s) - ", __func__); printk(x); } while(0)

/* Compat stuff. Ugly and getting worse with each 2.6 ("stable") release :( */

#ifdef HAVE_CLASS_SIMPLE

#define SNOOP_COMPAT_STRUCT_CLASS	struct class_simple
#define SNOOP_COMPAT_CLASS_CREATE	class_simple_create
#define SNOOP_COMPAT_CLASS_DESTROY	class_simple_destroy
#define SNOOP_COMPAT_DEVICE_CREATE	class_simple_device_add
#define SNOOP_COMPAT_DEVICE_DESTROY(a, b)	class_simple_device_remove(b)

#else

#define SNOOP_COMPAT_STRUCT_CLASS	struct class
#define SNOOP_COMPAT_CLASS_CREATE	class_create
#define SNOOP_COMPAT_CLASS_DESTROY	class_destroy

#ifdef HAVE_CLASS_DEVICE
#define SNOOP_COMPAT_DEVICE_DESTROY	class_device_destroy

#ifdef HAVE_CLASS_DEVICE_PARENT
#define SNOOP_COMPAT_DEVICE_CREATE(x, y...)	class_device_create(x, NULL, y)

#else /* ! HAVE_CLASS_DEVICE_PARENT */
#define SNOOP_COMPAT_DEVICE_CREATE	class_device_create

#endif /* HAVE_CLASS_DEVICE_PARENT */
#else /* ! HAVE_CLASS_DEVICE */

#define SNOOP_COMPAT_DEVICE_CREATE(x...)	(NULL)
#define SNOOP_COMPAT_DEVICE_DESTROY(x...)	do {} while (0)

#endif /* HAVE_CLASS_DEVICE */
#endif /* HAVE_CLASS_SIMPLE */

#ifdef HAVE_RCU_FDT

#define SNOOP_COMPAT_FDT_LOCK(x)	rcu_read_lock()
#define SNOOP_COMPAT_FDT_UNLOCK(x)	rcu_read_unlock()
#define SNOOP_COMPAT_MAXFDS(x)		files_fdtable(x)->max_fds

#else

#define SNOOP_COMPAT_FDT_LOCK(x)	spin_lock(x)
#define SNOOP_COMPAT_FDT_UNLOCK(x)	spin_unlock(x)
#define SNOOP_COMPAT_MAXFDS(x)		x->max_fds

#endif /* HAVE_RCU_FDT */

#ifdef HAVE_RCU_PIDHASH

#define SNOOP_COMPAT_PID_LOCK(x)	rcu_read_lock()
#define SNOOP_COMPAT_PID_UNLOCK(x)	rcu_read_unlock()

#else

#define SNOOP_COMPAT_PID_LOCK(x)	read_lock(x)
#define SNOOP_COMPAT_PID_UNLOCK(x)	read_unlock(x)

#endif /* HAVE_RCU_PIDHASH */

struct snoop_dev {
	atomic_t			open_count;
	struct cdev 			cdev;
	SNOOP_COMPAT_STRUCT_CLASS	*csnoop;
};

struct snoop {
	atomic_t		count;
	wait_queue_head_t	inq;
	struct list_head	handles;
	struct semaphore	sem;
	unsigned		head;
	unsigned		tail;
	char 			*buf;
	char			name[SNOOP_NAME_SIZE];
};

struct snoop_handle {
	struct list_head 		list;
	struct snoop			*snoop;
	struct file			*filp;
	const struct file_operations	*old_ops;
	struct file_operations		ops;
	char				name[SNOOP_NAME_SIZE];
};



#endif

